#
#$Id: dpm_sourceTasks.tcl,v 1.93.4.3 2011/01/26 22:47:35 sbanks Exp $
#
#dpm_sourcesTasks.tcl --
#
#This file contains procedures for the source operations
#
# Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008 Xilinx Inc. All Rights Reserved.
# This code is subject to the terms and conditions of the Xilinx
# End User License Agreement (version 2006.05)
#
namespace eval xilinx {
namespace eval Dpm {
   proc dpm_chFileRemoveHelper { iInterface } {
      set bResult true
      set iHdlFile 0
      dpm_returnOnFail { dpm_getInterface $iInterface "IHdlFile" iHdlFile }
      dpm_assert { {$iHdlFile} != 0 }
      dpm_getInterface $iHdlFile "IFile" iFile
      set fileOrigination 0
      if { [$iFile GetOrigination fileOrigination] == 0 } {
         return [dpm_Fail "Couldn't get the origination of file $_FileName"]
      } else {
         if { $fileOrigination != 0 } {
            return true
         }
      }      
      set sPath ""
      set sFilename ""
      dpm_returnOnFail { dpm_getIHdlFileName $iHdlFile sPath sFilename }
      set sFilepath [file normalize [file join $sPath $sFilename]]
      dpm_returnOnFail { dpm_getInterface $iInterface "IDesignUnit" iDesignUnit }
      AutoRelease::set iProject 0
      AutoRelease::set iDesign 0
      $iDesignUnit GetDesign iDesign
      if { $iDesign != 0 } {
         $iDesign GetProject iProject
      }
      set sExtent [file extension $sFilename]
      switch -- [string tolower $sExtent] {
         .elf {
            set bResult [dpm_xpsResetElfAssociations $iProject $sFilepath]
         }
         .xmp {
            set bResult [dpm_xpsUnlinkXpsFromProjnav $iProject $sFilepath]
         }   
         .vhd -
         .v {
            set lExtents { .sym }
            set bResult [dpm_sourceRemoveAuxFiles $iHdlFile $iProject $sPath $sFilename $lExtents]
         }
         .xaw {
            set lExtents { .v .vhd .sym }
            set bResult [dpm_sourceRemoveAuxFiles $iHdlFile $iProject $sPath $sFilename $lExtents]
         }
         .sgp {
            set root_name [file rootname $sFilepath]
            lappend lFiles $root_name.xise
            lappend lFiles ${root_name}_bb.v
            set bResult [dpm_sourceRemoveListofAuxFiles $iHdlFile $iProject $sFilename $lFiles ]
         }
         .xco {
            set sModuleName [file rootname $sFilename]
            set lCoreFiles ""
            dpm_coregenGetOutputList $sModuleName $iProject $sPath lCoreFiles
            set lFiles ""
            foreach lEntry $lCoreFiles {
               set sFile [lindex $lEntry 0]
               lappend lFiles $sFile
            }
            set root_name [file rootname $sFilepath]
            lappend lFiles $root_name.xise
            set bResult [dpm_sourceRemoveListofAuxFiles $iHdlFile $iProject $sFilename $lFiles ]
         }
         .tbw {
            set lExtents { .xwv .vhd .v }
            set bResult [dpm_sourceRemoveAuxFiles $iHdlFile $iProject $sPath $sFilename $lExtents]
         }
         .sch {
            set lExtents { .v .vhd .sym }
            set bResult [dpm_sourceRemoveAuxFiles $iHdlFile $iProject $sPath $sFilename $lExtents]
         }
         .ise -
         .xise {
            set root_name [file rootname $sFilepath]
            lappend lFiles $root_name.sgp
            lappend lFiles $root_name.xco
            lappend lFiles ${root_name}_bb.v
            set bResult [dpm_sourceRemoveListofAuxFiles $iHdlFile $iProject $sFilename $lFiles ]
         }
      }
      return $bResult
   }
   proc dpm_sourceRemoveAuxFiles {iHdlFile iProject sPath sFileName lExtents} {
      set sModuleName [file rootname $sFileName]
      set lFiles ""
      foreach extent $lExtents {
         set filename [file join $sPath "$sModuleName$extent"]
         lappend lFiles $filename
      }
      return [dpm_sourceRemoveListofAuxFiles $iHdlFile $iProject $sFileName $lFiles]
   }
   proc dpm_sourceRemoveListofAuxFiles { iMasterHdlFile iProject sFileName lFiles } {
      set bResult true
      set iFileManager 0
      dpm_returnOnFail { dpm_getInterface $iProject "IFileManager" iFileManager }
      dpm_Debug 1 "processing auxiliary files for $sFileName"
      foreach sFile $lFiles {
         dpm_Debug 11 "Removing $sFile"
         set iFileIter 0
         $iFileManager Find $sFile iFileIter
         if { $iFileIter != 0 } {
            if { ![ $iFileIter IsEnd ] } {
               set result [$iFileManager RemoveFile iFileIter]
               if { ! $result } {
                  dpm_Debug 1 "ERROR: could not remove auxiliary file, \"$sFile\", associated with \"$sFileName\"."
                  set bResult false
               }
            }
            _Release $iFileIter
         }
      }
      return $bResult
   }
   proc dpm_chCommonAddFileOkChecks { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set _extent [file extension $sMappedPath]
      if { ![dpm_getDesign $iInterface _iDesign] } {
         dpm_Fail "could not get iDesign interface from iProjectHelper"
         return ""
      }
      set sTopModuleType ""
      if { ! [_GetPMPropertyValue $_iDesign "PROP_Top_Level_Module_Type" sTopModuleType] } {
         dpm_Fail "getting Property value from design failed."
         return ""
      }
      if { [string equal $_extent ".xmp"] } {
         set _extent [string trim $_extent "."]
         set _isInstalled "TRUE"
         set _toolName "XPS"
         dpm_chGetRuntticValue "XPSBatchIsInstalled" _isInstalled
         if { [string equal $_isInstalled "FALSE"] } {
            return "$_toolName is not installed, adding $_extent files is not allowed."
         }
         variable aMyDeviceInfo
         set aMyDeviceInfo(HIERSETTING) "sub"
         if { ! [dpm_xpsVerifyDeviceSettings $sSourcePath $_iDesign "Add"] } {
            return "Errors were generated while trying to add the source. See the console for details."
         }
      }
      if { [string equal $_extent ".cdc"] } {
         set _extent [string trim $_extent "."]
         if { ![dpm_getInterface $iInterface "IProjectHelper" iProjectHelper] } {
            return ""
         }
         AutoRelease::set iProjectIInterface 0
         if { ! [$iProjectHelper GetDpmProject iProjectIInterface] } {
            dpm_Fail "can not get project interface"
            return ""
         }
         if { ![dpm_getInterface $iProjectIInterface "IProject" iProject] } {
            dpm_Fail "could not get iProject"
            return ""
         }
         if { ![dpm_replacePriorFileByExt $sSourcePath "$_extent" $iProject] } {
            return "multiple copies of $_extent files are not allowed."
         }
      }
      return ""
   }
   proc dpm_chAddFileOk { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set _extent [file extension $sMappedPath]
      return [dpm_chCommonAddFileOkChecks $sSourcePath $iInterface]
   }
   proc dpm_chAddCopyFileOk { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set sMBody "can not be added using Add Copy of Source ... use Add Source ... instead."
      set _extent [file extension $sMappedPath]
      if { $_extent == ".xmp" } {
         return "XMP files $sMBody"
      }
      if { $_extent == ".sgp" } {
         return "System Generator sources $sMBody"
      }
      if { $_extent == ".xise" } {
         return "ISE projects $sMBody"
      }
      return [dpm_chCommonAddFileOkChecks $sSourcePath $iInterface]
   }
   proc getFileType { iProject _strFileType } {
      $iProject GetDesign iDesign
      $iDesign GetDataStore iDataStore
      set iFileStore [$iDataStore GetInterface $::xilinx::Dpm::IFileTypeStoreID]
      set iFileType 0
      $iFileStore GetAllFileTypes iFTIter
      $iFTIter First
      while { ! [ $iFTIter IsEnd ] } {
         set iFileType [ $iFTIter CurrentItem ]
         set iDDId [$iFileType GetInterface $::xilinx::Dpm::IDefDataIdID]
         if { 0 != $iDDId } {
            set sId ""
            $iDDId GetID sId
            if { $sId == $_strFileType } { break }
         }
         $iFTIter Next
      }
      _Release $iDesign
      _Release  $iFTIter
      return $iFileType
   }
   proc dpm_xpsHandleFiles { iProject sFilePath sProcess } {
      AutoRelease::set iDesign 0
      AutoRelease::set iView 0
      AutoRelease::set iHdlFile 0
      set iNullFileType 0
      set iXpsHdlFileType [ getFileType $iProject "FILE_XPS_HDL" ]
      if { $iXpsHdlFileType == 0 } { 
         return [dpm_Fail "could not get iXpsHdlFileType"] 
      }
      set iXpsPlatgenFileType [ getFileType $iProject "FILE_PLATGEN_OPT" ]
      if { $iXpsPlatgenFileType == 0 } { 
         return [dpm_Fail "could not get iXpsPlatgenFileType"] 
      }
      set iXpsSimgenFileType [ getFileType $iProject "FILE_SIMGEN_OPT" ]
      if { $iXpsSimgenFileType == 0 } { 
         return [dpm_Fail "could not get iXpsSimgenFileType"] 
      }
      set iXpsLibgenFileType [ getFileType $iProject "FILE_LIBGEN_OPT" ]
      if { $iXpsLibgenFileType == 0 } { 
         return [dpm_Fail "could not get iXpsLibgenFileType"] 
      }
      set iXpsBitinitFileType [ getFileType $iProject "FILE_BITINIT_OPT" ]
      if { $iXpsBitinitFileType == 0 } { 
         return [dpm_Fail "could not get iXpsPlatgenFileType"] 
      }
      if { $sProcess == "platgen" } { set sViewName "VIEW_*PreSynthesis" }
      if { $sProcess == "simgen"  } { set sViewName "VIEW_PreSimulation" }
      if { $sProcess == "bitinit" } { set sViewName "VIEW_FPGAConfiguration" }
      dpm_returnOnFail { $iProject GetDesign iDesign }
      if { ![dpm_getViewByTypeInDesignUsingRegexp $iDesign $sViewName iView] } {
         return [dpm_Fail "Failed to get the Pre Synthesis view from design"]
      }
      set iXpsFileReader 0
      set iXpsFileReader [ _CreateInstance $::xilinx::Dpm::CXpsFileReaderCompID "IXpsFileReader"] 
      if { $iXpsFileReader == 0 } {
         return [ dpm_Fail "Failed to create CXpsFileReader component" ]
      }
      set result [ $iXpsFileReader ReadFile $sFilePath ]
      if { $result == 0 } {
         return [ dpm_Fail "Failed to read the input XML file $sFilePath: ReadFile method failed" ]           
      }
      set piStringIterator 0
      set result [ $iXpsFileReader GetDependencyFiles $sProcess piStringIterator ]
      if  { $result == 0 } {
         return [ dpm_Fail "Failed to get dependency files for $sProcess: GetDependencyFiles method failed" ]
      }
      for { $piStringIterator First } { ![ $piStringIterator IsEnd] } { $piStringIterator Next } { 
         set sDepFile [ $piStringIterator CurrentItem ]
         set sTriggerFileDir [ file dirname $sFilePath ]
         if { [ file pathtype $sDepFile ] == "relative"} {
            set sDepFile "$sTriggerFileDir/$sDepFile"
         }
         if { ![file exists $sDepFile] } { 
            continue 
         }
         set sExt ""
         set sExt [file extension $sDepFile]
         set sExt [string tolower $sExt]
         set sExt [string trimright $sExt]
         if { $sExt == "" } { 
            continue 
         }
         set iXpsFileType $iNullFileType 
         switch -- $sExt {
            ".v" -
            ".vhd" -
            ".vhdl" { set iXpsFileType $iXpsHdlFileType }
            ".opt"  { 
               if { [file rootname [ file tail $sDepFile ] ] == "platgen" } { set iXpsFileType $iXpsPlatgenFileType }
               if { [file rootname [ file tail $sDepFile ] ] == "simgen"  } { set iXpsFileType $iXpsSimgenFileType  }
               if { [file rootname [ file tail $sDepFile ] ] == "libgen"  } { set iXpsFileType $iXpsLibgenFileType  }
               if { [file rootname [ file tail $sDepFile ] ] == "bitinit" } { set iXpsFileType $iXpsBitinitFileType }
            }
         }
         if { $sExt == ".ucf" } { continue }
         if { ![dpm_addFileToProject $iProject $sDepFile $iXpsFileType iHdlFile] } {
            return [ dpm_Fail "Failed to add file $sDepFile to the project" ]
         }
         set iFile 0
         dpm_getInterface $iHdlFile "IFile" iFile
         if { $iFile == 0 } {
            return [ dpm_Fail "Failed to get iFile from IHdlFile" ]
         }
         dpm_returnOnFail { $iFile SetOrigination 2 }
         dpm_AddFileToView $iView $iFile
      }
      if { ![dpm_addFileToProject $iProject $sFilePath $iNullFileType iHdlFile] } {
         return [ dpm_Fail "Failed to add file $sFilePath to the project" ]
      }
      set iFile 0
      dpm_getInterface $iHdlFile "IFile" iFile
      if { $iFile == 0 } {
         return [ dpm_Fail "Failed to get IFile from IHdlFile" ]
      }
      dpm_returnOnFail { $iFile SetOrigination 2 }
      dpm_AddFileToView $iView $iFile
      if { $sProcess == "simgen"  } { 
         set sViewName "VIEW_Par"
         AutoRelease::set iView1 0
         if { ![dpm_getViewByTypeInDesignUsingRegexp $iDesign $sViewName iView1] } {
            return [dpm_Fail "Failed to get the Par view from design"]
         }
         if { $iView1 != 0 } {
            dpm_AddFileToView $iView1 $iFile
         }
      }
      return true
   }
   proc dpm_xpsAddSingleSimElf { sElfPath iInterface } {
      return [dpm_xpsAddSingleAssocElf $sElfPath $iInterface "sim"]
   }
   proc dpm_xpsAddSingleImpElf { sElfPath iInterface } {
      return [dpm_xpsAddSingleAssocElf $sElfPath $iInterface "imp"]
   }
   proc dpm_xpsAddSingleAssocElf { sElfPath iInterface { viewInfo "both" } } {
      set extent [file extension $sElfPath]
      if { $extent ne ".elf" } {
         return true ; # nothing to do
      }
      set iProjectHelper 0
      AutoRelease::set iProjectIInterface 0
      set iProject 0
      set iProjectHelper [$iInterface GetInterface $::xilinx::Dpm::IProjectHelperID]
      if { ! [$iProjectHelper GetDpmProject iProjectIInterface] } {
         return [dpm_Fail "can not get project interface"]
      }
      set iProject [$iProjectIInterface GetInterface $::xilinx::Dpm::IProjectID]
      set iElfFileType [ getFileType $iProject "FILE_ELF" ]
      if { $iElfFileType == 0 } { 
         return [dpm_Fail "could not get iElfFileType"] 
      }
      set iHdlFile 0
      if { ![dpm_addFileToProject $iProject $sElfPath $iElfFileType iHdlFile] } {
         puts "WARNING: adding associated elf file '$sElfPath' failed.\n   Verify elf file associations before bitstream generation."
      } else {
         set hint $ViewHint::AnyView
         set iDU [$iHdlFile GetInterface $::xilinx::Dpm::IDesignUnitID]
         set iPSH [$iProjectHelper GetInterface $::xilinx::Dpm::IProjSrcHelperID]
         $iPSH ForceDULocation $iDU 0 $hint
         dpm_forceBitgenOOD
      }
      return true
   }
   proc dpm_xpsAddAssocElfs { iProject iProjectHelper sSourcePath } {
      set iElfFileType [ getFileType $iProject "FILE_ELF" ]
      if { $iElfFileType == 0 } { 
         return [dpm_Fail "could not get iElfFileType"] 
      }
      dpm_xpsGetElfInfo $sSourcePath $iProject
      set sXmpProjDir [file dirname $sSourcePath]
      set spath [file normalize [file join $sXmpProjDir "pn_processor_info"]]
      if { [ catch { open $spath r } hppi ] } {
         puts "ERROR: unable to read $spath: $hppi"
         return false
      }
      set simelfs {}
      set impelfs {}
      while { [gets $hppi entry] >= 0 } {
         regexp {(.*)\|(.*) (.*)\,(.*)} $entry match xmpPath processor assoc elfPath
         if { $elfPath eq "bootloop" } {
            continue
         }
         if { $assoc eq "(simulation)" } {
            lappend simelfs $elfPath
         } else {
            lappend impelfs $elfPath
         }
      }
      close $hppi
      foreach simelf $simelfs {
         set assoc "sim"
         set index [ lsearch -exact $impelfs $simelf ]
         if { $index != -1 } {
            set assoc "both"
            set impelfs [ lreplace $impelfs $index $index ]
         }
         dpm_xpsAddSingleAssocElf $simelf $iProjectHelper $assoc
      }
      foreach impelf $impelfs {
         dpm_xpsAddSingleAssocElf $impelf $iProjectHelper "imp"
      }
      return true
   }
   proc dpm_chPostProcessAddFile { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set _extent [file extension $sMappedPath]
      if { $_extent == ".xmp" || $_extent == ".XMP" } {
         set iProjectHelper 0
         AutoRelease::set iProjectIInterface 0
         set iProject 0
         set iProjectHelper [$iInterface GetInterface $::xilinx::Dpm::IProjectHelperID]
         if { ! [$iProjectHelper GetDpmProject iProjectIInterface] } {
            return [dpm_Fail "can not get project interface"]
         }
         set iProject [$iProjectIInterface GetInterface $::xilinx::Dpm::IProjectID]
         if { $iProject == 0 } { return [dpm_Fail "could not get iProject"] }
         set sDir [ file dirname $sMappedPath ]
         set sRootName [ file rootname [ file tail $sMappedPath ] ]
         set sFilePath "$sDir/__xps/ise/$sRootName.xpssyn"
         if { [ file exists $sFilePath ] } {
            dpm_xpsHandleFiles $iProject $sFilePath "platgen"
         }
         set sFilePath "$sDir/__xps/ise/$sRootName.xpsupb"
         if { [ file exists $sFilePath ] } {
            dpm_xpsHandleFiles $iProject $sFilePath "bitinit"
         }
         set sFilePath "$sDir/__xps/ise/$sRootName.xpssim"
         if { [ file exists $sFilePath ] } {
            dpm_xpsHandleFiles $iProject $sFilePath "simgen"
         }
         dpm_xpsAddAssocElfs $iProject $iProjectHelper $sSourcePath
         puts "\nINFO: XMP source [file tail $sSourcePath] may not be referenced by your design."
         puts "      To implement this design either include the instantiation template in your top level design"
         puts "      or run the Generate Top HDL Source process, which will automatically create a top level HDL"
         puts "      which references this design. Use of Generate Top HDL Source is recommended if you want to"
         puts "      implement or analyze your processor design without any additional logic.\n"
      }
      if { $_extent == ".xco" } {
         set sCoreSourceDir [file dirname $sSourcePath]
         dpm_cgCreateCgpFile $sCoreSourceDir $sSourcePath
      }
      return true 
   }
   proc dpm_chPreProcessAddCopyFile { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set _extent [file extension $sMappedPath]
      if { $_extent == ".xco" } {
         set iProjectHelper [$iInterface GetInterface $::xilinx::Dpm::IProjectHelperID]
         AutoRelease::set iProjectIInterface 0
         if { ! [$iProjectHelper GetDpmProject iProjectIInterface] } {
            return [dpm_Fail "can not get project interface"]
         }
         set iProject [$iProjectIInterface GetInterface $::xilinx::Dpm::IProjectID]
         set iProject2 [$iProjectIInterface GetInterface $::xilinx::Dpm::IProject2ID]
         if { $iProject == 0 } { return [dpm_Fail "could not get iProject"] }
         if { $iProject2 == 0 } { return [dpm_Fail "could not get iProject2"] }
         set xisePath ""
         $iProject2 GetSourceProjectFileName xisePath
         set projDir [file normalize [file dirname $xisePath]]
         set projIPDir [file join $projDir "ipcore_dir"]
         if { ![file exists $projIPDir] || ![file isdirectory $projIPDir] } {
            dpm_Debug 1 "Add Copy: $projIPDir does not exist doing mkdir"
            file mkdir $projIPDir
         }
         set sFileName [file tail $sSourcePath]
         set sModuleName [file rootname $sFileName]
         set sSrcIPDir [file normalize [file dirname $sSourcePath]]
         dpm_Debug 11 "Add Copy: dest dir: $projIPDir, file: $sFileName, module: $sModuleName, source dir: $sSrcIPDir"
         set sCoreXiseFile [file join $sSrcIPDir $sModuleName.xise]
         if { [dpm_CompanionCoreGenISEExist $sCoreXiseFile] && $sSrcIPDir != $projIPDir } {
            dpm_Debug 11 "core ise file found: $sCoreXiseFile"
            set lCoreIseFiles ""
            set sCoreGiseFile [file join $sSrcIPDir $sModuleName.gise]
            lappend lCoreIseFiles $sCoreXiseFile
            lappend lCoreIseFiles $sCoreGiseFile
            set lCoreFiles ""
            if { ![dpm_coregenGetOutputList $sModuleName $iProject $sSrcIPDir lCoreFiles] } {
               puts "ERROR: could not find core output files for $sModuleName.xco"
               return false
            }
            foreach lItemInfo $lCoreFiles {
               set sItemPathName [lindex $lItemInfo 0]
               dpm_Debug 11 "processing core product $sItemPathName"
               set sItemTail [file tail $sItemPathName]
               if { [string equal $sItemPathName $sItemTail] } {
                  continue
               }
               if { $sSrcIPDir != $projIPDir } {
                  if { [catch { file copy -force $sItemPathName $projIPDir } sErrorMsg] } {
                     puts "ERROR: copying file $sItemPathName to $projIPDir failed: $sErrorMsg"
                     return false
                  }
               }
            }
            foreach lIseFile $lCoreIseFiles {
               if { [catch { file copy -force $lIseFile $projIPDir } sErrorMsg] } {
                  puts "WARNING: Copying supporting file $lIseFile to $projIPDir failed: $sErrorMsg"
               }
            }
         }
      }
      return true
   } ;#end of dpm_chPreProcessAddCopyFile
   proc dpm_chPostProcessAddCopyFile { sSourcePath iInterface } {
      set sMappedPath [string map {\\ /} $sSourcePath]
      set _extent [string tolower [file extension $sMappedPath]]
      if { $_extent == ".xco" || $_extent == ".xaw" || \
              $_extent == ".vhd" || $_extent == ".v" || $_extent == ".sch" } {
         dpm_Debug 11 "Called to handle $sSourcePath"
         set iProjectHelper [$iInterface GetInterface $::xilinx::Dpm::IProjectHelperID]
         AutoRelease::set iProjectIInterface 0
         if { ! [$iProjectHelper GetDpmProject iProjectIInterface] } {
            return [dpm_Fail "can not get project interface"]
         }
         set iProject [$iProjectIInterface GetInterface $::xilinx::Dpm::IProjectID]
         if { $iProject == 0 } { return [dpm_Fail "could not get iProject"] }
         set projSourceDir ""
         dpm_getProjectSourceDir $iProject projSourceDir
         set sFileName [file tail $sSourcePath]
         return [dpm_sourceAddCopyHelper $iProject $sMappedPath $projSourceDir $sFileName]
      }
      return true
   }
   proc dpm_setXmpElfAssociation { sSourcePath iInterface } {
      set iProj 0
      set iProj [ $iInterface GetInterface $::xilinx::Dpm::IProjectID ]
      if { $iProj == 0 } { return [dpm_Fail "could not get iProject interface"] }
      AutoRelease::set iDesign 0
      $iProj GetDesign iDesign
      set lProcessorInfo {}
      set lHdlFiles {}
      set lDesignFiles {}
      dpm_getFilesByTypeInDesign $iDesign "DESUT_XPS" lHdlFiles lDesignFiles
      foreach xmpFile $lDesignFiles {
         if { [ dpm_xpsGetProcessorList $xmpFile $iProj ] } {
            set sXmpProjDir [ file dirname $xmpFile ]
            set sppiPath [ file join $sXmpProjDir "pn_processor_list" ]
            if { [ file exists $sppiPath ] } {
               if { [catch { open $sppiPath r } hppi] } {
                  puts  "Error: Unable to open (read) $sppiPath - $hppi"
                  return false
               }
               while { [gets $hppi entry] >= 0 } {
                  set xmpinfo [lindex [split $entry ","] 0]
                  lappend lProcessorInfo "$xmpinfo"
               }
	       close $hppi
            }
         }
      }
      set dialogCompID "{065FCB28-F292-4cec-9AE6-C2C6E28E00DC}"
      if { ! [_LoadLibraries "libGuiI_Stub" "libCitI_CoreStub" "libUtilI_UtilStub"] } { 
         puts "ERROR: failed to create .elf File Association Dialog, could not load dependent libraries."
         return false 
      }
      set iStringListI 0
      if { ![_GetPluginInterface $dialogCompID $::xilinx::UtilI::IStringContainerID iStringListI] } {
         return [dpm_Fail "couldn't get the IStringContainer plugin interface"]
      }
      $iStringListI Add sSourcePath ; # add this first, its handled specially by the dialog.
      foreach pName $lProcessorInfo {
         $iStringListI Add pName
      }
      set iShowDialogI [$iStringListI GetInterface $::xilinx::GuiI::IShowDialogID]
      $iShowDialogI ShowDialog true 0
   }
   proc dpm_sourcePostAddOperation { sSourcePath _path _file } {
      set _extent [file extension $_file]
      set _RetVal true
      if { [string equal -nocase $_file ".cdc"] } {
         if { [file exists "xps_lock"] } {
            set _Msg "WARNING: Chipscope and XPS will both attempt to insert a BSCAN component
                         into the design. Since there is only one BSCAN location on the physical
                         device, this will cause an overmapping error in the Map process. To avoid
                         this, do not attempt to use both Chipscope and XPS cores in a design at
                         the same time."
            dpm_DisplayMessageBox $_Msg
         } 
         if { ![file exists "cdc_lock"] } {
            set _fileName "cdc_lock"
            dpm_Debug 1 "Creating the cdc lock file $_fileName"
            set _RetVal [dpm_xilTouch $_fileName]
         }
      }
      return $_RetVal
   }
   proc dpm_sourceCreateVerilog { sFilePath iNewSourceComp lOutputFileList } {
      upvar $lOutputFileList lOutputList
      set iNewSourceData 0
      dpm_returnOnFail { dpm_getInterface $iNewSourceComp "INewSourceData" iNewSourceData }
      set iProjectHelper 0
      dpm_returnOnFail { dpm_getProjectHelper $iNewSourceData iProjectHelper }
      if { [catch {open $sFilePath "w"} hFile] } {
         puts  "ERROR: Unable to create $sFilePath"
         puts  "Reason: $hFile"
         return false
      }
      set _t [clock seconds]
      set _ts [clock format $_t -format "%H:%M:%S %m/%d/%Y"]
      global xilinx::Dpm::INewSourceHdlDataID
      set _iHdlData [$iNewSourceComp GetInterface $INewSourceHdlDataID]
      dpm_assert { {$_iHdlData} != 0 }
      set _moduleName ""
      dpm_returnOnFail { $_iHdlData GetModuleName _moduleName }
      set _portIter 0
      $_iHdlData GetPorts _portIter
      puts $hFile "`timescale 1ns / 1ps"
      puts $hFile "//////////////////////////////////////////////////////////////////////////////////"
      puts $hFile "// Company: "
      puts $hFile "// Engineer: "
      puts $hFile "// "
      puts $hFile "// Create Date:    $_ts "
      puts $hFile "// Design Name: "
      puts $hFile "// Module Name:    $_moduleName "
      puts $hFile "// Project Name: "
      puts $hFile "// Target Devices: "
      puts $hFile "// Tool versions: "
      puts $hFile "// Description: "
      puts $hFile "//"
      puts $hFile "// Dependencies: "
      puts $hFile "//"
      puts $hFile "// Revision: "
      puts $hFile "// Revision 0.01 - File Created"
      puts $hFile "// Additional Comments: "
      puts $hFile "//"
      puts $hFile "//////////////////////////////////////////////////////////////////////////////////"
      puts $hFile "module ${_moduleName}("
      if { $_portIter != 0 } {
         $_portIter First
         while { ! [$_portIter IsEnd] } {
            set _iPortInfo [$_portIter CurrentItem]
            set _sPortDir ""
            set _portDir -1
            $_iPortInfo GetPortDirection _portDir
            switch -- $_portDir {
               0 { set _sPortDir "input" }
               1 { set _sPortDir "output" }
               2 { set _sPortDir "inout" }
               default { set _sPortDir "undef" }
            }
            puts -nonewline $hFile "    $_sPortDir "
            set _isScalar true
            $_iPortInfo IsScalar _isScalar
            if { ! $_isScalar } {
               set _range_a ""
               set _range_b ""
               $_iPortInfo GetBusRange _range_a _range_b
               set _range_a [expr $_range_a + 0]
               set _range_b [expr $_range_b + 0]
               puts -nonewline $hFile "\[${_range_a}:${_range_b}\] "
            }
            set _name ""
            $_iPortInfo GetName _name
            if { $_name != "" } {
               puts -nonewline $hFile "${_name}"
            }
            $_portIter Next
            if {! [$_portIter IsEnd] } {
            	puts $hFile ","
            } else {
            	puts $hFile ""
            }
         }
	      puts $hFile "    );"
      }
      puts $hFile ""
      puts $hFile ""
      puts $hFile "endmodule"
      flush $hFile
      close $hFile
      lappend lOutputList "$sFilePath"
      return true
   } ; # end dpm_sourceCreateVerilog ...
   proc dpm_sourceCreateVHDL { sFilePath iNewSourceComp lOutputFileList } {
      upvar $lOutputFileList lOutputList
      set iNewSourceData 0
      dpm_returnOnFail { dpm_getInterface $iNewSourceComp "INewSourceData" iNewSourceData }
      set iProjectHelper 0
      dpm_returnOnFail { dpm_getProjectHelper $iNewSourceData iProjectHelper }
      if { [catch {open $sFilePath "w"} hFile] } {
         puts  "ERROR: Unable to create $sFilePath"
         puts  "Reason: $hFile"
         return false
      }
      set _t [clock seconds]
      set _ts [clock format $_t -format "%H:%M:%S %m/%d/%Y"]
      global xilinx::Dpm::INewSourceHdlDataID
      set _iHdlData [$iNewSourceComp GetInterface $INewSourceHdlDataID]
      if { $_iHdlData == 0 } {
         return [dpm_Fail "interface passed to Create New Source code hook is invalid."]
      }
      set _entityName ""
      dpm_returnOnFail { $_iHdlData GetEntityName _entityName }
      set _archName ""
      dpm_returnOnFail { $_iHdlData GetModuleName _archName }
      dpm_Debug 11 "architecture returned is $_archName"
      set _portIter 0
      $_iHdlData GetPorts _portIter
      puts $hFile "----------------------------------------------------------"
      puts $hFile "-- Engineer: Lothar Miller"
      puts $hFile "-- "
      puts $hFile "-- Create Date:    $_ts "
      puts $hFile "-- Design Name: "
      puts $hFile "-- Module Name:    $_entityName - $_archName "
      puts $hFile "----------------------------------------------------------"
      puts $hFile "library IEEE;"
      puts $hFile "use IEEE.STD_LOGIC_1164.ALL;"
      puts $hFile "use IEEE.NUMERIC_STD.ALL;"
      puts $hFile ""
      puts $hFile "entity ${_entityName} is"
      if { $_portIter != 0 && ! [$_portIter IsEnd] } {
         set _preamble "    Port ( "
         while { ! [$_portIter IsEnd] } {
            set _typeinfo ""
            set _name ""
            set _portDir ""
            set _iPortInfo [$_portIter CurrentItem]
            set _sPortDir ""
            $_iPortInfo GetPortDirection _portDir
            switch -- $_portDir {
               0 { set _sPortDir "in" }
               1 { set _sPortDir "out" }
               2 { set _sPortDir "inout" }
               default { set _sPortDir "undef" }
            }
            set _isScalar true
            set _typeInfo ""
            $_iPortInfo IsScalar _isScalar
            set _signalType ""
            $_iPortInfo GetSignalTypeName _signalType
            if { $_isScalar } {
               set _typeInfo "$_signalType STD_LOGIC"
            } else {
               set _range_a ""
               set _range_b ""
               $_iPortInfo GetBusRange _range_a _range_b
               set _typeInfo "${_signalType} STD_LOGIC_VECTOR \(${_range_a} downto ${_range_b}\)"
            }
            set _name ""
            $_iPortInfo GetName _name
            if { $_name != "" } {
               puts -nonewline $hFile "${_preamble}${_name} : ${_sPortDir} ${_typeInfo}"
            }
            set _preamble ";\n           "
            $_portIter Next
         }
         puts $hFile ");"
      }
      puts $hFile "end ${_entityName};"
      puts $hFile ""
      puts $hFile "architecture ${_archName} of ${_entityName} is"
      puts $hFile ""
      puts $hFile "begin"
      puts $hFile ""
      puts $hFile ""
      puts $hFile "end ${_archName};"
      puts $hFile ""
      flush $hFile
      close $hFile
      lappend lOutputList "$sFilePath"
      return true
   } ; # end dpm_sourceCreateVHDL ...
   proc dpm_sourceCreateVHDLPackage { sFilePath iNewSourceComp lOutputFileList } {
      upvar $lOutputFileList lOutputList
      set iNewSourceData 0
      dpm_returnOnFail { dpm_getInterface $iNewSourceComp "INewSourceData" iNewSourceData }
      set iProjectHelper 0
      dpm_returnOnFail { dpm_getProjectHelper $iNewSourceData iProjectHelper }
      set pkgName [ file tail [ file rootname $sFilePath ] ]
      if { [catch {open $sFilePath "w"} hFile] } {
         puts  "ERROR: Unable to create $sFilePath"
         puts  "Reason: $hFile"
         return false
      }
      puts $hFile "--"
      puts $hFile "--	Package File Template"
      puts $hFile "--"
      puts $hFile "--	Purpose: This package defines supplemental types, subtypes, "
      puts $hFile "--		 constants, and functions "
      puts $hFile "--"
      puts $hFile "--   To use any of the example code shown below, uncomment the lines and modify as necessary"
      puts $hFile "--"
      puts $hFile ""
      puts $hFile "library IEEE;"
      puts $hFile "use IEEE.STD_LOGIC_1164.all;"
      puts $hFile ""
      puts $hFile "package $pkgName is"
      puts $hFile ""
      puts $hFile "-- type <new_type> is"
      puts $hFile "--  record"
      puts $hFile "--    <type_name>        : std_logic_vector\( 7 downto 0\);"
      puts $hFile "--    <type_name>        : std_logic;"
      puts $hFile "-- end record;"
      puts $hFile "--"
      puts $hFile "-- Declare constants"
      puts $hFile "--"
      puts $hFile "-- constant <constant_name>		: time := <time_unit> ns;"
      puts $hFile "-- constant <constant_name>		: integer := <value;"
      puts $hFile "--"
      puts $hFile "-- Declare functions and procedure"
      puts $hFile "--"
      puts $hFile "-- function <function_name>  \(signal <signal_name> : in <type_declaration>\) return <type_declaration>;"
      puts $hFile "-- procedure <procedure_name> \(<type_declaration> <constant_name>	: in <type_declaration>\);"
      puts $hFile "--"
      puts $hFile ""
      puts $hFile "end $pkgName;"
      puts $hFile ""
      puts $hFile "package body $pkgName is"
      puts $hFile ""
      puts $hFile "---- Example 1"
      puts $hFile "--  function <function_name>  \(signal <signal_name> : in <type_declaration>  \) return <type_declaration> is"
      puts $hFile "--    variable <variable_name>     : <type_declaration>;"
      puts $hFile "--  begin"
      puts $hFile "--    <variable_name> := <signal_name> xor <signal_name>;"
      puts $hFile "--    return <variable_name>; "
      puts $hFile "--  end <function_name>;"
      puts $hFile ""
      puts $hFile "---- Example 2"
      puts $hFile "--  function <function_name>  \(signal <signal_name> : in <type_declaration>;"
      puts $hFile "--                         signal <signal_name>   : in <type_declaration>  \) return <type_declaration> is"
      puts $hFile "--  begin"
      puts $hFile "--    if \(<signal_name> = '1'\) then"
      puts $hFile "--      return <signal_name>;"
      puts $hFile "--    else"
      puts $hFile "--      return 'Z';"
      puts $hFile "--    end if;"
      puts $hFile "--  end <function_name>;"
      puts $hFile ""
      puts $hFile "---- Procedure Example"
      puts $hFile "--  procedure <procedure_name>  \(<type_declaration> <constant_name>  : in <type_declaration>\) is"
      puts $hFile "--    "
      puts $hFile "--  begin"
      puts $hFile "--    "
      puts $hFile "--  end <procedure_name>;"
      puts $hFile " "
      puts $hFile "end $pkgName;"
      flush $hFile
      close $hFile
      lappend lOutputList "$sFilePath"
      return true
   } ; # end dpm_sourceCreateVHDLPackage ...
   proc dpm_sourceCreateVHDLLibrary { sFilePath iNewSourceComp lOutputFileList } {
      set iNewSourceData 0
      dpm_returnOnFail { dpm_getInterface $iNewSourceComp "INewSourceData" iNewSourceData }
      AutoRelease::set iProjectHelper 0
      dpm_returnOnFail { dpm_getProjectHelper $iNewSourceData iProjectHelper }
      AutoRelease::set _iIface 0
      dpm_returnOnFail { $iProjectHelper GetDpmProject _iIface }
      set _iSourceLibMgr [ $_iIface GetInterface $::xilinx::Dpm::ISourceLibraryManagerID ]
      set _sLibName [ file tail $sFilePath ]
      AutoRelease::set _iSourceLibrary 0
      dpm_returnOnFail { $_iSourceLibMgr CreateSourceLibrary _sLibName _iSourceLibrary }
      return true
   } ; # end dpm_sourceCreateVHDLLibrary
   proc dpm_chVetFileUIType { iInterface } {
      global xilinx::Dpm::IFilterID
      set iFilter [$iInterface GetInterface $IFilterID]
      set _family ""
      $iFilter GetValue "PM" _family
      set _fileType ""
      $iFilter GetValue "FileType" _fileType
      set _fileUIType ""
      $iFilter GetValue "FileUIType" _fileUIType
      set _SynthEnabled ""
      $iFilter GetValue "SYNTH_TOOL_IS_ENABLED" _SynthEnabled
      set _isSynthDisabled true
      if { [string equal "TRUE" $_SynthEnabled] } {
         set _isSynthDisabled false
      }
      set _synthTool ""
      $iFilter GetValue "SYNTHESIS_TOOL" _synthTool
      set _simTool ""
      $iFilter GetValue "SIMULATOR_TOOL" _simTool
      set _simLang "Mixed"
      if { [string match "*VHDL" $_simTool] } {
         set _simLang "VHDL"
      } 
      if { [string match "*Verilog" $_simTool] } {
         set _simLang "Verilog"
      }
      set _hasCores true
      set _isFpga true
      switch -- [string tolower $_family] {
         fpgacore   { set _hasCores false }
         acr2       -
         xbr        -
         xc9500     -
         xa9500xl   -
         xc9500xl   -
         xpla3      { set _isFpga false }
         default    { 
         }
      }
      set _usesNewFPE false
      switch -- [string tolower $_family] {
         virtex4    -
         virtex5    -
         spartan3a  -
         spartan3adsp { set _usesNewFPE true }
      }
      if { $_fileUIType == "FILEUI_VERILOG_TESTFIXTURE" || $_fileUIType == "FILEUI_VHDL_TESTBENCH" } {
         if { $_synthTool == "Synplify (VHDL)" && $_fileUIType == "FILEUI_VERILOG_TESTFIXTURE" } {
            return false
         }
         if { $_synthTool == "Synplify (Verilog)" && $_fileUIType == "FILEUI_VHDL_TESTFIXTURE" } {
            return false
         }
         return true
      }
      set _isInstalled "TRUE"
      switch -exact -- $_fileType {
         FILE_BMM -
         FILE_MEM {
            if { ! $_isFpga } { return false }
         }
         FILE_CDC {
            if { ! $_isFpga } { return false } 
         }
         FILE_COREGEN {
            if { ! $_hasCores || ! $_isFpga || $_isSynthDisabled } { 
               if {!$_hasCores} {dpm_Debug 38 "Disabling $_fileType in NSW for $_family Family"}
               if {!$_isFpga} {dpm_Debug 38 "Disabling $_fileType in NSW for $_family Family"}
               if {$_isSynthDisabled} {dpm_Debug 38 "Disabling $_fileType in NSW cause Synthesis is disabled"}
               return false 
            }
         }
         FILE_COREGENISE -
         FILE_XAW {
               return false 
         }
         FILE_IO_PINASSIGNMENTS {
            if { $_usesNewFPE && $_isSynthDisabled } { return false }
         }
         FILE_SCHEMATIC {
            if { [string match "Synplify*" $_synthTool] } { return false }
            if { $_isSynthDisabled} { return false }
         }
         FILE_VERILOG {
            if { [string equal "Synplify (VHDL)" $_synthTool] } { return false }
            if { $_isSynthDisabled} { return false }
         }
         FILE_VHDL {
            if { [string equal "Synplify (Verilog)" $_synthTool] } { return false }
            if { $_isSynthDisabled} { return false }
         }
         FILE_XPS {
            dpm_chGetRuntticValue "XPSBatchIsInstalled" _isInstalled
            if { [string equal $_isInstalled "FALSE"] || ! $_isFpga } { return false }
         }
         FILE_SYSGENISE {
            if { !$_isFpga } { return false }
            set sysgenPath ""
            if { ![dpm_sysgenGetExePath sysgenPath] } { return false }
         }         
      }
      return true
   } ; # end dpm_chVetFileUIType
   proc dpm_findExistingFile { iProject sFile iHdlFileHandle } {
      upvar $iHdlFileHandle iHdlFile
      set iFileManager 0
      dpm_getInterface $iProject "IFileManager" iFileManager
      set iHdlFileIter 0
      $iFileManager Find $sFile iHdlFileIter
      $iHdlFileIter First
      set iHdlFile [ $iHdlFileIter CurrentItem ]
      _AddReference $iHdlFile
      _Release $iHdlFileIter
   }
   proc dpm_processConflicts { iConflictIter sFile bIsUCF bAlreadyInProjectHandle } {
      upvar $bAlreadyInProjectHandle bAlreadyInProject
      set result true
      if { $iConflictIter == 0 } {
         return true
      }
      for { $iConflictIter First } { ! [$iConflictIter IsEnd] } { $iConflictIter Next } {
         set eNum 0
         set sMsg ""
         set iConflict [$iConflictIter CurrentItem]
         $iConflict GetErrorNum eNum
         $iConflict GetErrorString sMsg
         if { $eNum == 1 } {
            set bAlreadyInProject true
            continue
         } elseif { $eNum == 4 } {
            continue
         } elseif { $eNum == 5 && $bIsUCF } {
            set sMsg "Adding file $sFile failed, a UCF file has already been associated with the selected Design Unit. You must remove the current UCF file if you want to add a new UCF file for this Design Unit."
         } 
         puts "ERROR: $sMsg"
         set result false
      }
      return $result
   }
   proc dpm_chCreateNewSource { iNewSourceComp } {
      global xilinx::Dpm::INewSourceDataID
      global xilinx::Dpm::IProjectHelperReferenceID
      global xilinx::Dpm::IProjSrcHelperID
      global xilinx::Dpm::IViewNetworkHelpID
      global xilinx::Dpm::ISourceViewAssocID
      global xilinx::Dpm::IProjectID
      global xilinx::Dpm::IDefDataIdID
      set _fileHandle 0
      set iNewSourceData [$iNewSourceComp GetInterface $INewSourceDataID]
      set iProjectHelperRef [$iNewSourceComp GetInterface $IProjectHelperReferenceID]
      AutoRelease::set iProjectHelper 0
      dpm_returnOnFail { $iProjectHelperRef Get iProjectHelper }
      set _iProjSrcHelper [$iProjectHelper GetInterface $IProjSrcHelperID]
      AutoRelease::set iProjectIInterface 0
      dpm_returnOnFail { $iProjectHelper GetDpmProject iProjectIInterface }
      set iProject [$iProjectIInterface GetInterface $IProjectID]
      set _iDesign 0
      dpm_returnOnFail { $iProject GetDesign _iDesign }
      set _iFileUIType 0
      set res [$iNewSourceData GetFileUiType _iFileUIType]
      set _iFileUITypeId [$_iFileUIType GetInterface $IDefDataIdID]
      set _sFileUITypeId ""
      $_iFileUITypeId GetID _sFileUITypeId
      set sFileUILabel ""
      set _sCreate [SourceData::getFileCreateInfo $_sFileUITypeId]
      set sFileName ""
      set sFileDir ""
      $iNewSourceData GetFileDir sFileDir
      $iNewSourceData GetFileName sFileName
      set sFilePath $sFileName
      if { $sFileDir != "" } {
         set sMappedDir [string map {\\ /} $sFileDir]
         set sFilePath "$sMappedDir/$sFileName"
      }
      dpm_Debug 11 "Processing for file $sFilePath"
      set _doAdd 0
      $iNewSourceData GetAutoAddToProject _doAdd
      set lOutputFileList {}
      if { $_sCreate == "empty" } {
         if { $_sFileUITypeId == "FILEUI_BMM" } {
            if { $_doAdd && ![dpm_replacePriorFileByExt $sFilePath "bmm" $iProject ] } { return false }
         }
         set _fileStream [open $sFilePath "w"]
         close $_fileStream
         lappend lOutputFileList $sFilePath
      } else {
         if { $_sFileUITypeId == "FILEUI_CDC" } {
            if { $_doAdd && ![dpm_replacePriorFileByExt $sFilePath "cdc" $iProject] } { return false }
         }
         if { ! [$_sCreate $sFilePath $iNewSourceComp lOutputFileList] } {
            return false
         }
      }
      if { $_doAdd } {
         set _iDU 0
         $iNewSourceData GetAssociatedDu _iDU
         set _DUName ""
         if { $_iDU != 0 } {
            if { ![dpm_getDUName $_iDU aDUName] } {
               return false
            }
            set _DUName $aDUName(ModuleName)
         }
         set result true
         foreach _file $lOutputFileList {
            set bAlreadyInProject false
            set sMsg ""
            if { $_sFileUITypeId == "FILEUI_UCF" } {
               set _iViewNetHelper [$iProjectHelper GetInterface $IViewNetworkHelpID]
               set _iView 0
               $_iViewNetHelper GetViewNetworkByDu $_iDU $ViewHint::SynthesisOnly _iView
               set _TopDUName ""
               if { ![_GetViewProperty $_iView "TOPDESIGNUNIT" _TopDUName] } {
                  set sMsg "Adding file $sFileName failed, the selected Design Unit ($_DUName) is not the current Top Design Unit. Set the selected Design Unit as the Top Design Unit before adding an associated constraint file."
                  puts "ERROR: $sMsg"
                  set result false
               }
               set _Temp [split $_TopDUName "|"]
               set _TopDUName [lindex $_Temp 1]
               if { $result } {
                  if { ! [string equal $_DUName $_TopDUName] } {
                     set sMsg "Adding file $sFileName failed, the selected Design Unit ($_DUName) is not the current Top Design Unit ($_TopDUName). Set the selected Design Unit as the Top Design Unit before adding an associated constraint file."
                     puts "ERROR: $sMsg"
                     set result false
                  }
               }
               if { $result && $_iView != 0 } {
                  set _iSourceViewAssoc [$iProjectHelper GetInterface $ISourceViewAssocID]
                  set iConflictIter 0
                  set iHdlFile 0
                  $_iSourceViewAssoc AddSourceFileWithAssoc $_file $_iView $ViewHint::AnyView iConflictIter iHdlFile
                  set bIsUCF true
                  set result [dpm_processConflicts $iConflictIter $_file $bIsUCF bAlreadyInProject]
                  _Release $iConflictIter
                  if { $bAlreadyInProject } {
                     dpm_findExistingFile $iProject $_file iHdlFile
                  }
                  if { $iHdlFile == 0 } {
                     set result false
                  }
                  if { $result } {
                     set sPropName "PROP_lockPinsUcfFile"
                     _GetPMPropertyValue $_iDesign $sPropName sUcfPropValue
                     if { [string equal $sUcfPropValue ""] } {
                        dpm_setProperty $_iDesign $sPropName $_file 0
                     }
                  }
               } else {
                  set result false
               }
            } else { ; # NOT FILEUI_UCF
               set _iSourceViewAssoc [$iProjectHelper GetInterface $ISourceViewAssocID]
               set _iView 0
               set iConflictIter 0
               set iHdlFile 0
               if { $_sFileUITypeId == "FILEUI_VERILOG_TESTFIXTURE" || $_sFileUITypeId == "FILEUI_VHDL_TESTBENCH" } {
                  $_iSourceViewAssoc AddSourceFileWithAssoc \
                        $_file $_iView $ViewHint::SimulationOnly iConflictIter iHdlFile
               } else {
                  $_iSourceViewAssoc AddSourceFileWithAssoc \
                        $_file $_iView $ViewHint::AnyView iConflictIter iHdlFile
               }
               set bIsUCF false
               set result [dpm_processConflicts $iConflictIter $_file $bIsUCF bAlreadyInProject]
               _Release $iConflictIter
               if { $bAlreadyInProject } {
                  dpm_findExistingFile $iProject $_file iHdlFile
               }
               if { $iHdlFile == 0 } {
                  set result false
               }
               if { $result } {
                  set _iHdlDUIter 0
                  $iHdlFile getDesignUnits 0 _iHdlDUIter
                  set _myHint $ViewHintData::_viewHintInfo($_sFileUITypeId)
                  global xilinx::Dpm::IDesignUnitID
                  for { $_iHdlDUIter First } { ! [$_iHdlDUIter IsEnd] } { $_iHdlDUIter Next } {
                     set _iHdlDu [$_iHdlDUIter CurrentItem]
                     set _iDu [$_iHdlDu GetInterface $IDesignUnitID]
                     $_iProjSrcHelper ForceDULocation $_iDu $_iView $_myHint
                  }
                  _Release $_iHdlDUIter
                  dpm_chPostProcessAddFile $_file $iProjectHelper 
               }
               _Release $iHdlFile
            } ; # else not ucf
         } ; # foreach file outputlist
         if { $iProject != 0 } {
            $iProject Save
         }
         if { ! $result } {
            return $result
         }
      } ; # if doadd
      set _editMode 1
      set _DUInstance 0
      set _sourceDUNode 0
      if { $_sFileUITypeId == "FILEUI_UCF" } {
         dpm_editorCreateEditUcfFile $sFilePath $_iDesign
      } else {
         set _sEditor [SourceData::getFileEditInfo $_sFileUITypeId]
         if { $_sEditor != "none" && $_sEditor != ""} {
            dpm_Debug 10 "calling $_sEditor to edit $sFileName"
            $_sEditor $sFilePath $_DUInstance $_iDesign $_sourceDUNode $_editMode
         }
      }
      return true
   } ; # end dpm_chCreateNewSource
} ; # end namespace Dpm
} ; # end namespace xilinx
