#
#$Id: testbnch2.tcl,v 1.7 2010/02/12 21:07:33 sbanks Exp $
#
# testbench.tcl
#
# This file contains procedures and a main routine for writing a VHDL testbench
# from vhdtdtfi.
#
# Copyright (c) 2003, 2004, 2005, 2006, 2007 Xilinx Inc. All Rights Reserved.
# This code is subject to the terms and conditions of the Xilinx
# End User License Agreement (version 2006.05)
#
# This script is for use with vhdtdtfi.  Vhdtdtfi will write out the variables 
# below to the beginning of this file and save it into the project directory. 
# Then it will be run to create the output file.  If the debug option is set
# in vhdtdtfi, this file will remain, otherwise is will be deleted. 
# 
# In the future, a better interface will be created -- this one was designed 
# for simplicity and ease of implementation.  
#
#	set ModuleName "bounce8"
#	set ProjectName "prj"
#	set ProjectFile ""
#	set ProjectName "prj"
#	set OutputFileName "out"
#	set InputFileName "bounce8.v"
#	set TemplateFileName "testbench.tcl"
#	set ErrorFileName "err"
#	set LibLocation ""
#	
# 	# Port Definitions 
#	set PortName [ list  "xxxclock" "reset" "Q0" "Q1" "Q2" "Q3" "Q4" "Q5" "Q6" "Q7" ]
#	set LeftRange [ list  -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 ]
#	set RightRange [ list  -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 ]
#	set PortDir [ list  "I" "I" "O" "O" "O" "O" "O" "O" "O" "O" ]
#	set TotalPorts 10
#
#END_OF_HEADER

#
# Determine if the port name is in the form of a clock signal -- with clk or clock
# at the begining or end of the name
#
proc isClock {portname} {

   set portname  [ string tolower $portname ]

   if {[string first "clk" $portname] == 0} { return 1 }
   if {[string first "clock" $portname ] == 0} { return 1 }
   
   if {	[string first "clk" $portname ] > 0 && 
        [string first "clk" $portname ] == [string length $portname] - 3} { return 1 }
   if {	[string first "clock" $portname ] > 0 && 
        [string first "clock" $portname ] == [string length $portname] - 5} { return 1 }
   
   return 0
}
#proc isReset {portname} {

#   set portname  [ string tolower $portname ]

#   if {[string first "reset" $portname] == 0} { return 1 }
#   if {[string first "rst" $portname ] == 0} { return 1 }
   
#   if {	[string first "rst" $portname ] > 0 && 
#        [string first "rst" $portname ] == [string length $portname] - 3} { return 1 }
#   if {	[string first "reset" $portname ] > 0 && 
#        [string first "reset" $portname ] == [string length $portname] - 5} { return 1 }
   
#   return 0
#}
	
#
# Format a port or signal declaration with the correct direction (ports only), 
# std_logic type and bus indicators. 
#
proc formatDecl {Name LeftRange RightRange PortDir Type} {	

   set retval "$Name :"
   set others ""
   if { $PortDir eq "I" && $Type eq "signal" } { 
      set others " := '0'"
   }
   if {$Type eq "port"} {
      if { $PortDir eq "I"} { set retval "$retval IN " }
      if { $PortDir eq "O"} { set retval "$retval OUT " }
      if { $PortDir eq "B"} { set retval "$retval INOUT " }
   }
   if { $LeftRange == -1 && $RightRange == -1 } {
      set retval "$retval std_logic$others"
   } else { 
      if { $PortDir eq "I" && $Type eq "signal"} { 
         set others " := (others => '0')"
      }
      if {$LeftRange < $RightRange} {
         set retval "$retval std_logic_vector($LeftRange to $RightRange)$others"
      } else {
         set retval "$retval std_logic_vector($LeftRange downto $RightRange)$others"
      }
   }
   return $retval;
}

# 
# Execution begins here
#

set Entity [file tail [file rootname $OutputFileName]]

# create the output file, preparing to write out the template
set _fs [open $OutputFileName "w"]

# prepare date/time for comment header
set _t [clock seconds]
set _ts [clock format $_t -format "%H:%M:%S %m/%d/%Y"]

puts $_fs "--------------------------------------------------------------------------------"
puts $_fs "-- Company: "
puts $_fs "-- Engineer:"
puts $_fs "--"
puts $_fs "-- Create Date:   ${_ts}" 
puts $_fs "-- Design Name:   " 
puts $_fs "-- Module Name:   $OutputFileName"
puts $_fs "-- Project Name:  $ProjectName"
puts $_fs "-- "
#puts $_fs "-- Revision:"
#puts $_fs "-- Revision 0.01 - File Created"
#puts $_fs "-- Additional Comments:"
#puts $_fs "--"
#puts $_fs "-- Notes: AUS TESTBNCH2.TCL"
puts $_fs "--------------------------------------------------------------------------------"
puts $_fs "LIBRARY ieee;"
puts $_fs "USE ieee.std_logic_1164.ALL;"
puts $_fs "USE ieee.std_logic_unsigned.all;"
puts $_fs " " 
puts $_fs "ENTITY $Entity IS"
puts $_fs "END $Entity;"
puts $_fs " " 
puts $_fs "ARCHITECTURE behavior OF $Entity IS " 
puts $_fs " "

# Defines for clock periods
set firstClock ""
#set firstReset ""
set hasInputs 0
set hasOutputs 0
set hasBiDirs 0
set hasClocks 0
#set hasReset 0
	
#puts $_fs "    -- Component Declaration for the Unit Under Test (UUT)"
#puts $_fs " " 
puts $_fs "    COMPONENT $ModuleName"
puts $_fs "    PORT("

#
#  Get all the ports and write out in different formats
#
for {set i 0} {$i<$TotalPorts} {incr i} {
   puts -nonewline $_fs "         [formatDecl 	[lindex $PortName $i] \
						[lindex $LeftRange $i] \
						[lindex $RightRange $i] \
						[lindex $PortDir $i] "port" ]"

   # Gather data about the ports
   if { [lindex $PortDir $i] eq "I"} { set hasInputs 1 }
   if { [lindex $PortDir $i] eq "O"} { set hasOutputs 1 }
   if { [lindex $PortDir $i] eq "B"} { set hasBiDirs 1 }

   # Are there clocks?
   if { [isClock [lindex $PortName $i]] == 1 } {
      set hasClocks 1
      if {$firstClock eq ""} {
         set firstClock	[lindex $PortName $i]
      }
   }

   # Are there resets?
#   if { [isReset [lindex $PortName $i]] == 1 } {
#      set hasReset 1
#      if {$firstReset eq ""} {
#         set firstReset	[lindex $PortName $i]
#      }
#   }
		
   if { $i == $TotalPorts - 1} {
      puts $_fs ""
   } else {
      puts $_fs ";"
   }
}	
puts $_fs "        );"
puts $_fs "    END COMPONENT;"
puts $_fs "    "	

# do signal definitions, initialize inputs $_fs only
if {$hasInputs == 1} {
   puts $_fs ""
   puts $_fs "   --Inputs"
   for {set i 0} {$i<$TotalPorts} {incr i} {
      if {[lindex $PortDir $i] eq "I"} {
         puts $_fs "   signal [formatDecl 	[lindex $PortName $i] \
						[lindex $LeftRange $i] \
						[lindex $RightRange $i] \
						[lindex $PortDir $i] "signal"];"
      }
   }						
}

if {$hasBiDirs == 1} {
   puts $_fs ""
   puts $_fs "	--BiDirs"
   for {set i 0} {$i<$TotalPorts} {incr i} {
      if {[lindex $PortDir $i] eq "B"} {
         puts $_fs "   signal [formatDecl 	[lindex $PortName $i] \
						[lindex $LeftRange $i] \
						[lindex $RightRange $i] \
						[lindex $PortDir $i] "signal"];"
      }
   }						
}
	
if {$hasOutputs == 1} {
   puts $_fs ""
   puts $_fs " 	--Outputs"
   for {set i 0} {$i<$TotalPorts} {incr i} {
      if {[lindex $PortDir $i] eq "O"} {
         puts $_fs "   signal [formatDecl 	[lindex $PortName $i] \
						[lindex $LeftRange $i] \
						[lindex $RightRange $i] \
						[lindex $PortDir $i] "signal"];"
      }
   }						
}
	
if {$hasClocks == 1} {
   puts $_fs ""
   puts $_fs "   -- Clock period definitions"
   for {set i 0} {$i<$TotalPorts} {incr i} {
      set clk [lindex $PortName $i]
   }	
   puts $_fs " " 
}

if {$hasClocks == 0} {
   puts $_fs "   -- No clocks detected in port list. Replace <clock> below with "
   puts $_fs "   -- appropriate port name "
   puts $_fs " "
   puts $_fs "   constant <clock>_period : time := 10 ns;"
   puts $_fs " "
}
puts $_fs "BEGIN"
puts $_fs " " 

#
# Instantiate the uut
# Go over the ports and connect with signals -- "port1 => port1"
#
puts $_fs "	-- Instantiate the Unit Under Test (UUT)"
puts $_fs "   uut: $ModuleName PORT MAP ("
for {set i 0} {$i<$TotalPorts} {incr i} {
   puts -nonewline  $_fs "          [lindex $PortName $i] => [lindex $PortName $i]"
   if { $i == $TotalPorts - 1} {
      puts $_fs ""
   } else {
      puts $_fs ","
   }
}	
puts $_fs "        );"
	
# Write the clock processes -- 1 per found clock
if {$hasClocks == 1} {
   puts $_fs ""
   puts $_fs "   -- Clock definitions"
   for {set i 0} {$i<$TotalPorts} {incr i} {
      set clk [lindex $PortName $i]
      if {[isClock $clk] == 1} {
         puts $_fs "		$clk <= not $clk after 10 ns;"
         puts $_fs " " 
      }
   }	
}

# Write the testbench process
puts $_fs ""
puts $_fs "   -- Stimulus process"
puts $_fs "   stim_proc: process"
puts $_fs "   begin		"
#puts $_fs "      -- hold reset state for 100 ns."
puts $_fs "      -- do sth for 100 ns."
puts $_fs "      wait for 100 ns;	"
puts $_fs ""
if { $hasClocks == 1} { 
   puts $_fs "      wait until rising_edge($firstClock);"
}
puts $_fs ""
puts $_fs "      -- insert stimulus here "
#puts $_fs "" 
puts $_fs "      wait;"
puts $_fs "   end process;"
puts $_fs ""
puts $_fs "END;"

